@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## A real Try Catch Block Implementation Using Perl Filter - ~/lib/Nice/Try.pm
-## Version v1.0.6
+## Version v1.0.7
 ## Copyright(c) 2021 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2020/05/17
-## Modified 2021/06/05
+## Modified 2021/06/06
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -25,7 +25,7 @@ BEGIN
     use Scalar::Util ();
     use List::Util ();
     use Want ();
-    our $VERSION = 'v1.0.6';
+    our $VERSION = 'v1.0.7';
     our $ERROR;
     our( $CATCH, $DIED, $EXCEPTION, $FINALLY, $HAS_CATCH, @RETVAL, $SENTINEL, $TRY, $WANTARRAY );
 }
@@ -675,13 +675,14 @@ if( \$INC{'threads.pm'} && !CORE::exists( \$INC{'forks.pm'} ) )
 }
 CORE::local \$Nice::Try::WANT;
 CORE::local ( \$Nice::Try::EXCEPTION, \$Nice::Try::DIED, \@Nice::Try::RETVAL, \@Nice::Try::VOID );
+CORE::local \$Nice::Try::WANTARRAY = CORE::wantarray;
 CORE::local \$Nice::Try::TRY = CORE::sub
 {
-    CORE::do __TRY_OPEN_NL__{ __BLOCK_PLACEHOLDER__ };__TRY__CLOSE_NL__
+    \@Nice::Try::LAST_VAL = CORE::do __TRY_OPEN_NL__{ __BLOCK_PLACEHOLDER__ };__TRY__CLOSE_NL__
+    CORE::return( \@Nice::Try::LAST_VAL ) if( !CORE::defined( \$Nice::Try::WANTARRAY ) && CORE::scalar( \@Nice::Try::LAST_VAL ) );
     CORE::return( \$Nice::Try::VOID[0] = \$Nice::Try::SENTINEL );
 };
 __FINALLY_BLOCK__ CORE::local \$Nice::Try::HAS_CATCH = $has_catch_clause;
-CORE::local \$Nice::Try::WANTARRAY = CORE::wantarray;
 EOT
             if( !$self->{is_tied} && !$self->{dont_want} )
             {
@@ -779,7 +780,8 @@ EOT
             }
             else 
             {
-                \@Nice::Try::VOID = &\$Nice::Try::TRY;
+                &\$Nice::Try::TRY;
+                \$Nice::Try::RETVAL[0] = \$Nice::Try::LAST_VAL if( CORE::defined( \$Nice::Try::LAST_VAL ) );
             }
         }
     };
@@ -907,7 +909,8 @@ EOT
                 my $catch_code = <<EOT;
             CORE::local \$Nice::Try::CATCH = CORE::sub
             {
-                CORE::do __CATCH_OPEN_NL__{ __BLOCK_PLACEHOLDER__ }; __CATCH__CLOSE_NL__
+                \@Nice::Try::LAST_VAL = CORE::do __CATCH_OPEN_NL__{ __BLOCK_PLACEHOLDER__ }; __CATCH__CLOSE_NL__
+                CORE::return( \@Nice::Try::LAST_VAL ) if( !CORE::defined( \$Nice::Try::WANTARRAY ) && CORE::scalar( \@Nice::Try::LAST_VAL ) );
                 CORE::return \$Nice::Try::SENTINEL;
             };
             
@@ -967,7 +970,7 @@ EOT
                 } 
                 else 
                 {
-                    \@Nice::Try::VOID = \&\$Nice::Try::CATCH;
+                    \&\$Nice::Try::CATCH;
                 }
             }
 EOT
@@ -1106,90 +1109,66 @@ if( CORE::defined( \$Nice::Try::WANTARRAY ) and
       ( Scalar::Util::blessed( \$Nice::Try::RETVAL[0] ) && !\$Nice::Try::RETVAL[0]->isa( 'Nice::Try::SENTINEL' ) ) 
     ) ) 
 {
-    if( CORE::defined( \$Nice::Try::RETVAL[0] ) && \$Nice::Try::RETVAL[0] eq '__NEXT__' )
+    unless( CORE::defined( \$Nice::Try::BREAK ) )
     {
-        \$Nice::Try::BREAK = 'next';
-    }
-    elsif( CORE::defined( \$Nice::Try::RETVAL[0] ) && \$Nice::Try::RETVAL[0] eq '__LAST__' )
-    {
-        \$Nice::Try::BREAK = 'last';
-    }
-    elsif( CORE::defined( \$Nice::Try::RETVAL[0] ) && \$Nice::Try::RETVAL[0] eq '__REDO__' )
-    {
-        \$Nice::Try::BREAK = 'redo';
-    }
-    elsif( CORE::defined( \$Nice::Try::WANT ) && CORE::length( \$Nice::Try::WANT ) )
-    {
-        if( \$Nice::Try::WANT eq 'LIST' )
+        if( CORE::defined( \$Nice::Try::WANT ) && CORE::length( \$Nice::Try::WANT ) )
         {
-            CORE::return( \@Nice::Try::RETVAL );
-        }
-        elsif( \$Nice::Try::WANT eq 'VOID' )
-        {
-            if( CORE::defined( \$Nice::Try::RETVAL[0] ) && \$Nice::Try::RETVAL[0] eq '__NEXT__' )
+            if( \$Nice::Try::WANT eq 'LIST' )
             {
-                \$Nice::Try::BREAK = 'next';
+                CORE::return( \@Nice::Try::RETVAL );
             }
-            elsif( CORE::defined( \$Nice::Try::RETVAL[0] ) && \$Nice::Try::RETVAL[0] eq '__LAST__' )
+            elsif( \$Nice::Try::WANT eq 'VOID' )
             {
-                \$Nice::Try::BREAK = 'last';
+                if( CORE::defined( \$Nice::Try::RETVAL[0] ) && \$Nice::Try::RETVAL[0] eq '__NEXT__' )
+                {
+                    \$Nice::Try::BREAK = 'next';
+                }
+                elsif( CORE::defined( \$Nice::Try::RETVAL[0] ) && \$Nice::Try::RETVAL[0] eq '__LAST__' )
+                {
+                    \$Nice::Try::BREAK = 'last';
+                }
+                elsif( CORE::defined( \$Nice::Try::RETVAL[0] ) && \$Nice::Try::RETVAL[0] eq '__REDO__' )
+                {
+                    \$Nice::Try::BREAK = 'redo';
+                }
             }
-            elsif( CORE::defined( \$Nice::Try::RETVAL[0] ) && \$Nice::Try::RETVAL[0] eq '__REDO__' )
+            elsif( \$Nice::Try::WANT eq 'OBJECT' )
             {
-                \$Nice::Try::BREAK = 'redo';
+                CORE::return( \$Nice::Try::RETVAL[0] );
+            }
+            elsif( \$Nice::Try::WANT eq 'REFSCALAR' )
+            {
+                CORE::return( \\\$Nice::Try::RETVAL[0] );
+            }
+            elsif( \$Nice::Try::WANT eq 'SCALAR' )
+            {
+                CORE::return( \$Nice::Try::RETVAL[0] );
+            }
+            elsif( \$Nice::Try::WANT eq 'BOOLEAN' )
+            {
+                CORE::return( \$Nice::Try::RETVAL[0] );
+            }
+            elsif( \$Nice::Try::WANT eq 'CODE' )
+            {
+                CORE::return( \$Nice::Try::RETVAL[0] );
+            }
+            elsif( \$Nice::Try::WANT eq 'HASH' )
+            {
+                CORE::return( { \@Nice::Try::RETVAL } );
+            }
+            elsif( \$Nice::Try::WANT eq 'ARRAY' )
+            {
+                CORE::return( \\\@Nice::Try::RETVAL );
+            }
+            elsif( \$Nice::Try::WANT eq 'GLOB' )
+            {
+                CORE::return( \$Nice::Try::RETVAL[0] );
             }
         }
-        elsif( \$Nice::Try::WANT eq 'OBJECT' )
+        else
         {
-            CORE::return( \$Nice::Try::RETVAL[0] );
+            CORE::return( \$Nice::Try::WANTARRAY ? \@Nice::Try::RETVAL : \$Nice::Try::RETVAL[0] );
         }
-        elsif( \$Nice::Try::WANT eq 'REFSCALAR' )
-        {
-            CORE::return( \\\$Nice::Try::RETVAL[0] );
-        }
-        elsif( \$Nice::Try::WANT eq 'SCALAR' )
-        {
-            CORE::return( \$Nice::Try::RETVAL[0] );
-        }
-        elsif( \$Nice::Try::WANT eq 'BOOLEAN' )
-        {
-            CORE::return( \$Nice::Try::RETVAL[0] );
-        }
-        elsif( \$Nice::Try::WANT eq 'CODE' )
-        {
-            CORE::return( \$Nice::Try::RETVAL[0] );
-        }
-        elsif( \$Nice::Try::WANT eq 'HASH' )
-        {
-            CORE::return( { \@Nice::Try::RETVAL } );
-        }
-        elsif( \$Nice::Try::WANT eq 'ARRAY' )
-        {
-            CORE::return( \\\@Nice::Try::RETVAL );
-        }
-        elsif( \$Nice::Try::WANT eq 'GLOB' )
-        {
-            CORE::return( \$Nice::Try::RETVAL[0] );
-        }
-    }
-    else
-    {
-        CORE::return( \$Nice::Try::WANTARRAY ? \@Nice::Try::RETVAL : \$Nice::Try::RETVAL[0] );
-    }
-}
-elsif( scalar( \@Nice::Try::VOID ) )
-{
-    if( CORE::defined( \$Nice::Try::VOID[0] ) && \$Nice::Try::VOID[0] eq '__NEXT__' )
-    {
-        \$Nice::Try::BREAK = 'next';
-    }
-    elsif( CORE::defined( \$Nice::Try::VOID[0] ) && \$Nice::Try::VOID[0] eq '__LAST__' )
-    {
-        \$Nice::Try::BREAK = 'last';
-    }
-    elsif( CORE::defined( \$Nice::Try::VOID[0] ) && \$Nice::Try::VOID[0] eq '__REDO__' )
-    {
-        \$Nice::Try::BREAK = 'redo';
     }
 }
 EOT
@@ -1199,11 +1178,11 @@ EOT
         # my $token = PPI::Token->new( "; \{ $try_catch_code \}" ) || die( "Unable to create token" );
         # XXX 2021-05-11 (Jacques): Need to remove blocks so that next or last statements can be effective.
         my $envelop = <<EOT;
-; CORE::local \$Nice::Try::BREAK;
+; CORE::local( \$Nice::Try::BREAK, \@Nice::Try::LAST_VAL );
 \{
 __TRY_CATCH_CODE__
 \}
-if( \$Nice::Try::BREAK )
+if( CORE::defined( \$Nice::Try::BREAK ) )
 {
     if( \$Nice::Try::BREAK eq 'next' )
     {
@@ -1218,6 +1197,8 @@ if( \$Nice::Try::BREAK )
         CORE::redo;
     }
 }
+no warnings 'void';
+CORE::scalar( \@Nice::Try::LAST_VAL ) > 1 ? \@Nice::Try::LAST_VAL : \$Nice::Try::LAST_VAL[0];
 EOT
         $envelop =~ s/\n/ /gs unless( $self->{debug_code} );
         $envelop =~ s/__TRY_CATCH_CODE__/$try_catch_code/;
@@ -1308,7 +1289,18 @@ sub _process_loop_breaks
         }
         elsif( $class eq 'PPI::Token::Word' && $content =~ /^(?:CORE\::)?(?:GLOBAL\::)?(next|last|redo)$/ )
         {
-            $e->set_content( qq{CORE::return( '__} . uc( $1 ) . qq{__' )} );
+            $self->_message( 5, "Found loop keyword '$content'." );
+            # $e->set_content( qq{CORE::return( '__} . uc( $1 ) . qq{__' )} );
+            # $e->set_content( q{$Nice::Try::BREAK='__} . uc( $1 ) . qq{__' ); return;} );
+            my $break_code = q{($Nice::Try::BREAK='} . $1 . qq{') ) && return;};
+            my $break_doc = PPI::Document->new( \$break_code, readonly => 1 );
+            my $new_elem = $break_doc->first_element;
+            # $self->_browse( $new_elem );
+            $new_elem->remove;
+            $self->_message( 5, "New element is object '", sub{ overload::StrVal( $new_elem ) }, "' -> $new_elem" );
+            # Not yet implemented as of 2021-05-11 dixit PPI, so we use a hack to make it available anyhow
+            $e->replace( $new_elem );
+            $self->_message( 5, "Loop keyword now replaced with '$e'." );
         }
         elsif( $class eq 'PPI::Statement::Break' )
         {
@@ -1331,7 +1323,8 @@ sub _process_loop_breaks
             {
                 # We add our special return value. Notice that we use 'return' and not 
                 # 'CORE::return'. See below why.
-                my $break_code = qq{return( '__} . uc( $word1 ) . qq{__' )};
+                # my $break_code = qq{return( '__} . uc( $word1 ) . qq{__' )};
+                my $break_code = q{($Nice::Try::BREAK='} . $word1 . qq{') && return};
                 # e.g. next if( $i == 2 );
                 # next and if are both treated as 'word' by PPI
                 if( scalar( @$words ) > 1 )
@@ -1344,6 +1337,7 @@ sub _process_loop_breaks
                     $break_code .= ';'
                 }
                 $self->_message( 5, "Replacing this node with: $break_code" );
+                
                 my $break_doc = PPI::Document->new( \$break_code, readonly => 1 );
                 my $new_elem = $break_doc->first_element;
                 # $self->_browse( $new_elem );
@@ -1351,10 +1345,11 @@ sub _process_loop_breaks
                 $self->_message( 5, "New element is object '", sub{ overload::StrVal( $new_elem ) }, "' -> $new_elem" );
                 # Not yet implemented as of 2021-05-11 dixit PPI, so we use a hack to make it available anyhow
                 $e->replace( $new_elem );
+                $self->_message( 5, "Updated element now is '$e' for class '", $e->class, "' and parent class '", $e->parent->class, "'." );
                 # 2021-05-12 (Jacques): I have to do this workaround, because weirdly enough
                 # PPI (at least with PPI::Node version 1.270) will refuse to add our element
                 # if the 'return' word is 'CORE::return' so, we add it without and change it after
-                $new_elem->first_element->set_content( 'CORE::return' );
+                # $new_elem->first_element->set_content( 'CORE::return' );
                 # $self->_message( 5, "return litteral value is: ", $new_elem->first_element->content );
             }
             next;
@@ -1575,7 +1570,8 @@ sub _serialize
         my $self    = ref $_[0] ? shift : return undef;
         # If our object and the other are not of the same class, PPI refuses to replace 
         # to avoid damages to perl code
-        my $other = _INSTANCE(shift, ref $self) or return undef;
+        # my $other = _INSTANCE(shift, ref $self) or return undef;
+        my $other = shift;
         # die "The ->replace method has not yet been implemented";
         $self->parent->__replace_child( $self, $other );
         1;
@@ -1697,7 +1693,7 @@ Also since version 1.0.0, L<Nice::Try> is context aware:
 
 =head1 VERSION
 
-    v1.0.6
+    v1.0.7
 
 =head1 DESCRIPTION
 
@@ -1944,7 +1940,9 @@ will produces:
 
     Got here with Oops
 
-In group 3, L<TryCatch> was working wonderfully, but was relying on L<Devel::Declare> which was doing some esoteric stuff and eventually the version 0.006020 broke L<TryCatch> and there seems to be no intention of correcting this breaking change.
+In group 3, L<TryCatch> was working wonderfully, but was relying on L<Devel::Declare> which was doing some esoteric stuff and eventually the version 0.006020 broke L<TryCatch> and there seems to be no intention of correcting this breaking change. Besides, L<Devel::Declare> is now marked as deprecated and its use is officially discouraged.
+
+L<TryCatch> does not support any C<finally> block.
 
 In group 4, there is L<Syntax::Keyword::Try>, which is a great alternative if you do not care about exception class filter (it supports variable assignment since 2020-08-01 with version 0.18).
 
